function solvePointWkt(array) {
  let features = [];
  for (let item of array) {
    features.push(
      turf.point(
        item.wkt.split(",").map((item) => Number(item)),
        item
      )
    );
  }
  return turf.featureCollection(features);
}
function solvePointLnglat(array) {
  let features = [];
  for (let item of array) {
    features.push(turf.point([item.longitude, item.latitude], item));
  }
  return turf.featureCollection(features);
}
function solvePolygonsWkt(array) {
  let features = [];
  for (let item of array) {
    let wktarr = item.wkt.split(";");
    if (wktarr.length < 4) {
      // return null;
    } else {
      for (let i = 0; i < wktarr.length; i++) {
        wktarr[i] = wktarr[i].split(",");
        for (let j = 0; j < wktarr[i].length; j++) {
          wktarr[i][j] = Number(wktarr[i][j]);
        }
      }
      features.push(turf.polygon([wktarr], item));
    }
  }
  return turf.featureCollection(features);
}
function solveSingleLineArray(array) {
  let features = [];
  features.push(turf.lineString(array));
  return turf.featureCollection(features);
}
function solveMultiLineArray(array){
  let features = [];
  for(let item of array){
    features.push(turf.lineString(item.lineArray));
  }
  return turf.featureCollection(features);
}
function solvePolygonsBuffer(array) {
  let features = [];
  for (let item of array) {
    var point = turf.point([item.longitude, item.latitude], item);
    let radius = item.radius * 0.001;
    let buffered = turf.buffer(point, radius, {
      units: "kilometers",
    });
    features.push(buffered);
  }
  return turf.featureCollection(features);
}

// 道路line图层
export function addOrUpdateRoadLine(map, layerName, arr){
  let geo = solveMultiLineArray(arr)
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "line",
      source: layerName,
      layout:{
        "line-join": 'round',
        'line-cap':'round'
      },
      paint: {
        "line-color": "rgba(120, 185, 255, 0.7)",
        "line-width": 8,
        'line-blur':2,
      },
    });
  }
}

// 缓冲区图层
export function addOrUpdatePointBuffer(map, layerName, arr) {
  let geo = solvePolygonsBuffer(arr);
  if (geo) {
    if (!map.getSource(layerName)) {
      map.addSource(layerName, {
        type: "geojson",
        data: geo,
      });
    } else {
      map.getSource(layerName).setData(geo);
    }
    if (!map.getLayer(layerName)) {
      map.addLayer({
        id: layerName,
        source: layerName,
        type: "fill",
        paint: {
          "fill-color": "rgba(29, 91, 196, 0.39)",
          "fill-opacity": 0.6,
        },
      });
    }
  }
  // ---内环
  let arr1 = JSON.parse(JSON.stringify(arr))
  for(let item of arr1){
    item.radius = item.radius * 0.6
  }
  let geo1 = solvePolygonsBuffer(arr1);
  if (geo1) {
    if (!map.getSource(`${layerName}1`)) {
      map.addSource(`${layerName}1`, {
        type: "geojson",
        data: geo1,
      });
    } else {
      map.getSource(`${layerName}1`).setData(geo1);
    }
    if (!map.getLayer(`${layerName}1`)) {
      map.addLayer({
        id: `${layerName}1`,
        source: `${layerName}1`,
        type: "fill",
        paint: {
          "fill-color": "rgba(29, 91, 196, 0.39)",
          "fill-opacity": 0.8,
        },
      });
    }
  }
  // ---- 
  let geoText = solvePointLnglat(arr);
  if (geoText) {
    if (!map.getSource(`${layerName}Text`)) {
      map.addSource(`${layerName}Text`, {
        type: "geojson",
        data: geoText,
      });
    } else {
      map.getSource(`${layerName}Text`).setData(geoText);
    }
    if (!map.getLayer(`${layerName}Text`)) {
      map.addLayer({
        id: `${layerName}Text`,
        type: "symbol",
        source: `${layerName}Text`,
        layout: {
          "text-field": ["get", "centerText"],
          "text-allow-overlap": true,
          "text-ignore-placement": true,
          "text-size": 24,
          "text-font": ["Arial Unicode MS Regular","Open Sans Regular"],
          'text-pitch-alignment':'map',
        },
        paint: {
          "text-color": "rgba(255, 255, 255, 1)",
          "text-halo-color": "#205582",
          "text-halo-width": 2,
          "text-halo-blur": 0.7,
        },
      });
    }
  }
}

// 点位icon图层
export function addOrUpdatePointIcon(map, layerName, arr, iconName) {
  let geo = solvePointLnglat(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "symbol",
      source: layerName,
      layout: {
        "icon-image": iconName,
        "icon-size": 0.7,
        "icon-offset": [0, 0],
        "icon-allow-overlap": true,
      },
    });
  }
}

// 相位空放区域polygon
export function addOrUpdatePhasePolygon(map, layerName, arr) {
  let geo = solvePolygonsWkt(arr);
  if (geo) {
    if (!map.getSource(layerName)) {
      map.addSource(layerName, {
        type: "geojson",
        data: geo,
      });
    } else {
      map.getSource(layerName).setData(geo);
    }
    if (!map.getLayer(layerName)) {
      map.addLayer({
        id: layerName,
        source: layerName,
        type: "fill",
        paint: {
          "fill-color": "#00ff3c",
          "fill-opacity": 0.6,
        },
      });
    }
  }
}

// 直左冲突点位热力图
export function addOrUpdateConflictPoint(map, layerName, arr) {
  let geo = solvePointLnglat(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "circle",
      source: layerName,
      paint: {
        "circle-radius": 30,
        "circle-blur": 1,
        "circle-color": "#efba0a",
        "circle-pitch-alignment": "map",
      },
    });
  }
}

// 可移动基站图标
export function addOrUpdateMobile(map, layerName, arr) {
  let geo = solvePointLnglat(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "symbol",
      source: layerName,
      layout: {
        "icon-image": "mobile",
        "icon-size": 0.7,
        "icon-offset": [0, 0],
      },
    });
  }
}

// 相机图标图层
export function addOrUpdateHomeCameraLayer(map, layerName, arr, callback) {
  let geo = solvePointWkt(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "symbol",
      source: layerName,
      layout: {
        "icon-image": "homeCamera",
        "icon-size": 0.4,
        "icon-offset": [0, 0],
        "icon-allow-overlap": true,
      },
    });
    map.on("click", layerName, callback);
  }
}

// 更新zoom值较小时的车辆轨迹point
export function addOrUpdateVehicle(map, layerName, arr) {
  let geo = solvePointLnglat(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "circle",
      source: layerName,
      paint: {
        "circle-blur": 1,
        "circle-color": "#06f4fe",
        "circle-pitch-alignment": "map",
      },
    });
  }
}

// 行人区域人数映射fill图层
export function addOrUpdateWaitingPolygon(map, layerName, arr) {
  let geo = solvePolygonsWkt(arr);
  console.log("geo...", geo);
  if (geo) {
    if (!map.getSource(layerName)) {
      map.addSource(layerName, {
        type: "geojson",
        data: geo,
      });
    } else {
      map.getSource(layerName).setData(geo);
    }
    if (!map.getLayer(layerName)) {
      map.addLayer({
        id: layerName,
        source: layerName,
        type: "fill",
        paint: {
          "fill-color": [
            "match",
            ["get", "val"],
            1,
            "#009c3e",
            2,
            "#009c3e",
            3,
            "#009c3e",
            "#009c3e",
          ],
          "fill-opacity": 0.6,
        },
      });
    }
  }
}

// 中间层级车辆图片图层
export function addOrUpdateVehiclePic(map, layerName, arr) {
  let geo = solvePointLnglat(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer(
      {
        id: layerName,
        type: "symbol",
        source: layerName,
        layout: {
          "icon-allow-overlap": true,
          "icon-ignore-placement": true,
          "icon-size": 0.2,
          "icon-image": ["concat", "car", ["get", "originalType"]],
          "icon-pitch-alignment": "map",
          "icon-rotate": ["get", "courseAngle"],
          "icon-rotation-alignment": "map",
        },
      },
      "vehicle3D"
    );
  }
}

// 慢行交通行人路线
export function addOrUpdateSlowLines(map, layerName, arr) {
  let geo = solveSingleLineArray(arr);
  if (!map.getSource(layerName)) {
    map.addSource(layerName, {
      type: "geojson",
      data: geo,
    });
  } else {
    map.getSource(layerName).setData(geo);
  }
  if (!map.getLayer(layerName)) {
    map.addLayer({
      id: layerName,
      type: "line",
      source: layerName,
      paint: {
        "line-color": "red",
        "line-width": 2,
      },
    });
  }
}
