import axios from 'axios'
import { tansParams, blobValidate } from "./tools";
import cache from './cache'
import { saveAs } from 'file-saver'

let downloadLoadingInstance;
// 是否显示重新登录
export let isRelogin = { show: false };

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8'
// 创建axios实例
const service = axios.create({
    // 超时
    timeout: 1000000
})

// request拦截器
service.interceptors.request.use(config => {
    // console.log('axios request...', config);
    // 是否需要防止数据重复提交
    const isRepeatSubmit = (config.headers || {}).repeatSubmit === false
    // get请求映射params参数
    if (config.method === 'get' && config.params) {
        let url = config.url + '?' + tansParams(config.params);
        url = url.slice(0, -1);
        config.params = {};
        config.url = url;
    }
    if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
        const requestObj = {
            url: config.url,
            data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
            time: new Date().getTime(),
        }
        const sessionObj = cache.session.getJSON('sessionObj')
        if (sessionObj === undefined || sessionObj === null || sessionObj === '') {
            cache.session.setJSON('sessionObj', requestObj)
        } else {
            const s_url = sessionObj.url;                  // 请求地址
            const s_data = sessionObj.data;                // 请求数据
            const s_time = sessionObj.time;                // 请求时间
            const interval = 1000;                         // 间隔时间(ms)，小于此时间视为重复提交
            if (s_data === requestObj.data && requestObj.time - s_time < interval && s_url === requestObj.url) {
                const message = '数据正在处理，请勿重复提交';
                console.warn(`[${s_url}]: ` + message)
                return Promise.reject(new Error(message))
            } else {
                cache.session.setJSON('sessionObj', requestObj)
            }
        }
    }
    return config
}, error => {
    console.log(error)
    Promise.reject(error)
})

// 响应拦截器
service.interceptors.response.use(res => {
    console.log('axios response...', res);
    // 未设置状态码则默认成功状态
    const code = res.data.code || 200;
    // 二进制数据则直接返回
    if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
        // 给二进制数据添加对应的请求参数信息，避免接收到不需要的数据
        // return { data: res.data, params: JSON.parse(res.config.data) }
        return res.data
    }
    if (code === 401) {
        if (!ELEMENT.$_overTime) {
            ELEMENT.$_overTime = ELEMENT.Notification.error({
                title: "失败",
                message: "code 401, 3秒后退出登录",
            });
            setTimeout(() => {
                window.location.href = "/";
            }, 3000);
        }
        return Promise.reject('无效的会话，或者会话已过期，请重新登录。')
    }
    if (code !== 200) {
        ELEMENT.Message.error(res.data.message || res.data.status)
    } else {
        if (res.config.showMsg) {
            ELEMENT.Message.success(res.data.message || res.data.status)
        }
    }

    return res.data
})

// 通用下载方法
export function download(url, params, filename) {
    downloadLoadingInstance = ELEMENT.Loading.service({ text: "正在下载数据，请稍候", spinner: "el-icon-loading", background: "rgba(0, 0, 0, 0.7)", })
    return service.get(url, params, {
        transformRequest: [(params) => { return tansParams(params) }],
        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
        responseType: 'blob'
    }).then(async (data) => {
        const isLogin = await blobValidate(data);
        if (isLogin) {
            const blob = new Blob([data])
            const link = document.createElement('a');
            link.href = window.URL.createObjectURL(blob);
            link.download = filename;
            link.click();
            // saveAs(blob, filename)
        } else {
            console.log('isNotBlob');
        }
        downloadLoadingInstance.close();
    }).catch((r) => {
        console.error(r)
        downloadLoadingInstance.close();
    })
}

export function getBlob(url) {
    return service.get(url, {
        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
        responseType: 'blob',
    }).then(async (data) => {
        const blob = new Blob([data])
        return blob
    })
}

export default service
